<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://ForwardMX.io
 * @since             1.0.0
 * @package           Forwardmx_Email_Alias
 *
 * @wordpress-plugin
 * Plugin Name:       ForwardMX Email Alias
 * Plugin URI:        https://ForwardMX.io/wordpress
 * Description:       Create email forward aliases for all your users automagically with ForwardMX.io
 * Version:           1.0.0
 * Author:            ForwardMX
 * Author URI:        https://ForwardMX.io
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       forwardmx-email-alias
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-forwardmx-email-alias-activator.php
 */
function activate_forwardmx_email_alias() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-forwardmx-email-alias-activator.php';
	Forwardmx_Email_Alias_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-forwardmx-email-alias-deactivator.php
 */
function deactivate_forwardmx_email_alias() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-forwardmx-email-alias-deactivator.php';
	Forwardmx_Email_Alias_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_forwardmx_email_alias' );
register_deactivation_hook( __FILE__, 'deactivate_forwardmx_email_alias' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-forwardmx-email-alias.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_forwardmx_email_alias() {

	$plugin = new Forwardmx_Email_Alias();
	$plugin->run();

}
run_forwardmx_email_alias();

add_action( 'admin_menu', 'forwardmx_menu' );
add_action( 'admin_init', 'register_forwardmx_settings' );

/**
 * Enqueue scripts
 */
function forwardmx_email_enqueue_scripts() {
    wp_enqueue_script( 'maven-pro-font', '//fonts.googleapis.com/css?family=Maven+Pro' );
}
add_action( 'admin_enqueue_scripts', 'forwardmx_email_enqueue' );

/**
 * Enqueue styles
 */
function forwardmx_email_enqueue_styles() {
    wp_enqueue_style( 'forwardmx-email-admin-style', plugin_dir_path( __FILE__ ) . '/public/css/forwardmx-email-admin-css' );
}
add_action( 'admin_print_styles', 'forwardmx_email_enqueue_styles' );



function register_forwardmx_settings() {
  register_setting( 'forwardmx-options', 'api_key' );
	register_setting( 'forwardmx-options', 'domains' );
	register_setting( 'forwardmx-options', 'domain' );
	register_setting( 'forwardmx-options', 'roles' );
}

function forwardmx_menu() {
	add_submenu_page( 'users.php', 'ForwardMX Email Alias Options', 'ForwardMX Alias',
		'manage_options', 'forwardmx-options', 'forwardmx_options' );
}

function forwardmx_options() {
	if ( !current_user_can( 'manage_options' ) )  {
		wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
	}
	?>

	<?php
		$forwardmx_your_domain = str_replace("http://","",str_replace("https://","",str_replace("www.","",get_site_url())));

		if($_GET['setup_domain'] == 'true'){
			$domain_data = wp_remote_get( 'https://forwardmx.io/api/domain/create/?key=' . get_option('api_key') . '&domain=' . $forwardmx_your_domain )['body'];
			if(strpos($domain_data, '"error"') === false){
				update_option('domain', $forwardmx_your_domain);
			}
		}
	?>
	<div class="wrap" style="font-family:'Maven Pro'">
		<header style="background-color:#232c3b; padding-top:12px; padding-right:20px; padding-bottom:12px; padding-left:20px;">
			<h1 style="color:white;font-family:'Maven Pro'">
				<img src="<?php echo plugin_dir_url(__FILE__) . 'public/logo.svg' ?>" alt="ForwardMX logo" style="height:18px; margin-right:4px;">
				ForwardMX &nbsp; <small>Email Alias Options</small>
			</h1>
		</header>
		<form method="post" action="options.php">
	    <?php settings_fields( 'forwardmx-options' ); ?>
	    <?php do_settings_sections( 'forwardmx-options' ); ?>
	    <table class="form-table">
	      <tr valign="center">
	        <th scope="row">ForwardMX <a href="https://forwardmx.io/billing/">API key</a></th>
	        <td><input type="text" name="api_key"
										 style="width:100%;"
										 value="<?php echo esc_attr( get_option('api_key') ); ?>"
										 placeholder="ASD-123123-ASDASD" /></td>
	      </tr>
				<?php if(get_option('api_key') != ''){ ?>
					<?php
						$domain_data = wp_remote_get( 'https://forwardmx.io/api/domains/?key=' . get_option('api_key') )['body'];
						update_option('domains', $domain_data);

						if($domain_data != '{"error":"API Key not found"}'){
					?>
						<tr>
							<th scope="row" valign="center">Choose Domain</th>
							<td>
								<select name="domain">
								<?php
								$data = json_decode($domain_data);
								$domain_found = false;
								foreach($data as $domain){
									if($domain->domain == $forwardmx_your_domain){
										$domain_found = true;
									}
									?>
									<option value="<?php echo esc_url($domain->domain) ?>" <?php if(get_option('domain') == $domain->domain){ echo ' selected'; } ?>>
										<?php echo esc_url($domain->domain) ?></option>
									<?php
								}
								?>
								</select>
						</tr>

						<tr>
							<th scope="row" valign="center" style="padding-top:6px">Autosetup roles</th>
							<td>
								<?php
									$roles = get_editable_roles();
									foreach($roles as $name => $role){
										?>
										<label>
											<input type="checkbox" name="roles[<?php $name ?>]" style="width:16px; height:16px; zoom: 1.5;"
														 value="<?php echo esc_attr($name) ?>"<?php if(in_array($name, get_option("roles"))){ echo ' checked'; } ?>>
														 <?php echo $esc_html(role["name"]) ?> &nbsp;
										</label>
										<?php
									}
								?>
								<br><br>
							</td>
						</tr>
					<?php }} ?>
	    </table>

	    <?php submit_button(); ?>
		</form>
	</div>
<?php
}

add_action('manage_users_columns','forwardmx_modify_users_columns');
function forwardmx_modify_users_columns($column_headers) {
  $column_headers['forwardmx'] = 'ForwardMX';
  return $column_headers;
}
add_action('manage_users_custom_column', 'forwardmx_add_users_column', 10, 3);
function forwardmx_add_users_column($value, $column_name, $user_id) {
	if($column_name == 'forwardmx'){
		$meta = get_user_meta($user_id, '_forwardmx', true);
		if($meta == ''){
			return "<big>&times;</big>";
		} else {
	  	return $meta;
		}
	}
}
add_action('admin_footer-users.php', 'fmxea_bulk_options');
add_action( 'admin_action_add_forwardmx', 'fmxea_bulk_add' );
add_action( 'admin_action_remove_forwardmx', 'fmxea_bulk_remove' );

function fmxea_bulk_options() {
	if(get_option('api_key') != ''){
  ?>
  <script type="text/javascript">
    jQuery(document).ready(function() {
      jQuery('<option>').val('add_forwardmx').text('<?php _e('Add ForwardMX Alias')?>').appendTo("select[name='action']");
      jQuery('<option>').val('add_forwardmx').text('<?php _e('Add ForwardMX Alias')?>').appendTo("select[name='action2']");
			jQuery('<option>').val('remove_forwardmx').text('<?php _e('Remove ForwardMX Alias')?>').appendTo("select[name='action']");
			jQuery('<option>').val('remove_forwardmx').text('<?php _e('Remove ForwardMX Alias')?>').appendTo("select[name='action2']");
    });
  </script>
  <?php
	}
}
function fmxea_bulk_add(){
	foreach($_REQUEST['users'] as $user_id){
		$user = get_userdata( filter_var( $user_id, FILTER_SANITIZE_NUMBER_INT) );
		wp_remote_get( 'https://forwardmx.io/api/alias/create/?key=' . get_option('api_key') . '&domain=' . get_option('domain') . '&alias=' . sanitize_text_field($user->user_login) . '&destination=' . sanitize_email($user->user_email))['body'];

		$value = $user->user_login . "@" . get_option('domain');
		update_user_meta( $user_id, '_forwardmx', sanitize_text_field($value) );
	}
}
function fmxea_bulk_remove(){
	foreach($_REQUEST['users'] as $user_id){
		$user = get_userdata( filter_var( $user_id, FILTER_SANITIZE_NUMBER_INT) );
		wp_remote_get( 'https://forwardmx.io/api/alias/destroy/?key=' . get_option('api_key') . '&domain=' . get_option('domain') . '&alias=' . sanitize_text_field($user->user_login))['body'];
		update_user_meta( $user_id, '_forwardmx', '' );
	}
}

add_action( 'user_register', 'fmxea_registration_save', 10, 1 );

function fmxea_registration_save( $user_id ) {
	if(in_array(sanitize_text_field($_POST['role']), get_option('roles'))){
		wp_remote_get( 'https://forwardmx.io/api/alias/create/?key=' . get_option('api_key') . '&domain=' . get_option('domain') . '&alias=' . sanitize_user($_POST['user_login']) . '&destination=' . sanitize_email($_POST['email']))['body'];
		$value = sanitize_user($_POST['user_login']) . "@" . get_option('domain');
		update_user_meta( $user_id, '_forwardmx',  sanitize_text_field($value));
	}
}

add_action( 'delete_user', 'fmxea_registration_delete' );

function fmxea_registration_delete( $user_id ) {

	if(get_user_meta($user_id, '_forwardmx', true) != ''){
		$user = get_userdata( $user_id );
		wp_remote_get( 'https://forwardmx.io/api/alias/destroy/?key=' . get_option('api_key') . '&domain=' . get_option('domain') . '&alias=' . sanitize_text_field($user->user_login))['body'];
		$value =  $user->user_login . "@" . get_option('domain');
		update_user_meta( $user_id, '_forwardmx',  sanitize_text_field($value));
	}

}

add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), 'fmxea_add_action_links' );

function fmxea_add_action_links ( $links ) {
	$mylinks = array(
 		'<a href="' . admin_url( 'users.php?page=forwardmx-options' ) . '">Settings</a>',
	);
	return array_merge( $mylinks, $links );
}

function fmxea_admin_notice__add_api() {
    ?>
    <div class="notice notice-warning is-dismissible">
        <p>Please add an API key on the <a href="<?php admin_url( 'users.php?page=forwardmx-options' ) ?>">ForwardMX settings page</a></p>
    </div>
    <?php
}

function fmxea_admin_notice__invalid_api() {
    ?>
    <div class="notice notice-error is-dismissible">
        <p>Your API key is not valid, please check and re-enter it on the <a href="<?php admin_url( 'users.php?page=forwardmx-options' ) ?>">ForwardMX settings page</a></p>
    </div>
    <?php
}

if(get_option('api_key') == ''){
	add_action( 'admin_notices', 'fmxea_admin_notice__add_api' );
}
if(get_option('domains') == '{"error":"API Key not found"}'){
	add_action( 'admin_notices', 'fmxea_admin_notice__invalid_api' );
}
